-- Unit Frames Code

local PartyFrame = nil
local PetsFrame = nil
local MeFrame = nil
local GroupFrames = { }

local MaxBuffs = 6

local function initialConfigFunction(frame)

	frame.buttons = { }
	frame:RegisterForClicks("AnyUp")	
	
	if (not Healium.ShowPercentage) then frame.HPText:Hide() end
	table.insert(Healium_Frames, frame)
	

	-- configure buff frames
	frame.buffs = { }	
	local framename = frame:GetName()	
	for i=1, MaxBuffs, 1 do
		local buffFrame = getglobal(framename.."_Buff"..i)
		local name = buffFrame:GetName()
		buffFrame.icon = getglobal(name.."Icon")
		buffFrame.cooldown = getglobal(name.."Cooldown")
		buffFrame.count = getglobal(name.."Count")
		buffFrame.border = getglobal(name.."Border")
		buffFrame.id = i
		frame.buffs[i] = buffFrame
	end
	
	-- configure heal button frames
	for i=1, Healium_MaxButtons, 1 do
		local button = getglobal(framename.."_Heal"..i)
		frame.buttons[i] = button

		if (Healium_VarsLoaded) then
			-- set spell attribute for button
			local spell = HealiumDropDownButton[i]
			local id
			for k=1, Healium_MaxClassSpells, 1 do
				if (spell == Healium_Spell.Name[k]) then id = Healium_Spell.ID[k] end
			end		
			
			Healium_UpdateButtonSpell(button, spell, id)		
			
			-- set icon for button
			local texture = HealiumDropDownButtonIcon[i]	
			--print(i)
			Healium_UpdateButtonIcon(button, texture)
			
			--if (texture) then DEFAULT_CHAT_FRAME:AddMessage("t= " .. texture)	end
		
			if (i > Healium.ButtonCount) then 
				button:Hide()
			else
				button:Show()
			end

		end
	end
--	Healium_DebugPrint("Inital Config")
end

local function SetHeaderAttributes(frame)
	frame.initialConfigFunction = initialConfigFunction
	
--	frame:SetAttribute("showParty", "true")
--	frame:SetAttribute("showRaid", "true")
	frame:SetAttribute("showPlayer", "true")
	frame:SetAttribute("maxColumns", 1)
	frame:SetAttribute("columnAnchorPoint", "LEFT")
	frame:SetAttribute("point", "TOP")
	frame:SetAttribute("template", "HealiumUnitFrames_ButtonTemplate")
	frame:SetAttribute("templateType", "Button")
	frame:SetAttribute("unitsPerColumn", 5) 
end

local function CreateHeader(TemplateName, FrameName, ParentFrame)
	local f = CreateFrame("Frame", FrameName, ParentFrame, TemplateName)
	f:SetMovable(true)
	f:EnableMouse(true)
	f:SetPoint("TOPLEFT", ParentFrame, "BOTTOMLEFT")
	SetHeaderAttributes(f)			
	return f
end

local function CreateUnitFrame(FrameName, Caption, IsPet, Group)
	local uf = CreateFrame("Frame", FrameName, UIParent, "HealiumUnitFrameTemplate")
	getglobal(FrameName .. "_Caption"):SetText(Caption)		
	return uf
end

local function CreatePetHeader(FrameName, ParentFrame)
	local h = CreateHeader("SecureGroupPetHeaderTemplate", FrameName, ParentFrame)
	h:SetAttribute("filterOnPet", "true")
	h:SetAttribute("unitsPerColumn", 40) -- allow pets frame to show more than 5	
	h:SetAttribute("showSolo", "true")	
	h:SetAttribute("showRaid", "true")	
	h:SetAttribute("showParty", "true")	
	h:Show()
	return h
end

local function CreateGroupHeader(FrameName, ParentFrame, Group)
	local h = CreateHeader("SecureGroupHeaderTemplate", FrameName, ParentFrame)
	h:SetAttribute("groupFilter", Group)	
	h:SetAttribute("showRaid", "true")	
	h:Show()
	return h
end

local function CreatePartyHeader(FrameName, ParentFrame)
	local h = CreateHeader("SecureGroupHeaderTemplate", FrameName, ParentFrame)
	h:SetAttribute("showSolo", "true")		
	h:Show()
	return h
end

local function CreateMeHeader(FrameName, ParentFrame)
	local h = CreateHeader("SecureGroupHeaderTemplate", FrameName, ParentFrame)
	h:SetAttribute("showSolo", "true")		
	h:SetAttribute("nameList", UnitName("Player"))
	h:Show()
	return h
end

local function CreateGroupUnitFrame(FrameName, Caption, Group)
	local uf = CreateUnitFrame(FrameName, Caption)
	local h = CreateGroupHeader(FrameName .. "_Header", uf, Group)
	return uf
end

local function CreatePetUnitFrame(FrameName, Caption)
	local uf = CreateUnitFrame(FrameName, Caption)
	local h = CreatePetHeader(FrameName .. "_Header", uf)
	return uf
end

local function CreateMeUnitFrame(FrameName, Caption)
	local uf = CreateUnitFrame(FrameName, Caption)
	local h = CreateMeHeader(FrameName .. "_Header", uf)
	return uf
end

local function CreatePartyUnitFrame(FrameName, Caption)
	local uf = CreateUnitFrame(FrameName, Caption)
	local h = CreatePartyHeader(FrameName .. "_Header", uf)
	return uf
end

function HealiumUnitFrames_OnMouseDown(self, button)
	if button == "LeftButton" then
		self:StartMoving()	
	end
	
	if button == "RightButton" then
		ToggleDropDownMenu(1, nil, HealiumMenu, self, 0, 0)	
	end
end

function HealiumUnitFrames_OnMouseUp(self, button)
	if button == "LeftButton" then
		self:StopMovingOrSizing()	
	end
	
	if button == "RightButton" then
	
	end	
end

function HealiumUnitFrames_ShowHideFrame(self, show)
	if self == PartyFrame then
		Healium.ShowPartyFrame = show
		return
	end
	
	if self == PetsFrame then
		Healium.ShowPetsFrame = show
		return
	end
	
	if self == MeFrame then
		Healium.ShowMeFrame = show
		return
	end
	
	for i,j in ipairs(GroupFrames) do
		if self == j then
			Healium.ShowGroupFrames[i] = show
		end
	end	
end

function HealiumUnitFrames_Button_OnLoad(self)
	self.initialConfigFunction = HealiumUnitFrames_Header_Initial_Config
	self:RegisterForDrag("RightButton")
	self.HealthBar = getglobal(self:GetName() .. "_HealthBar")
	self.name = getglobal(self:GetName() .. "_Name")
	self.HPText = getglobal(self:GetName() .. "_HPText")
	self.name:SetWidth(80)	
end

function HealiumUnitFrames_Button_OnShow(self)
	local unit = self:GetAttribute("unit")
	if unit then
		self.TargetUnit = unit 
		-- normally this was set in Healium_CreateButton() but the unit attribute isn't set till just prior to showing the button, so we fix it here!		
		for i=1, Healium_MaxButtons, 1 do		
			self.buttons[i].TargetUnit = unit
		end
		
		self.name:SetText(strupper(UnitName(unit)))
		
		if not Healium_Units[unit] then
			Healium_Units[unit] = { }
		end
		
		table.insert(Healium_Units[unit], self)

		for i =1, MaxBuffs, 1 do
			self.buffs[i].unit = unit
		end

		Healium_UpdateUnitHealth(unit, self)				
		Healium_UpdateHealthBarColor(self)
		Healium_UpdateBuffs(unit, self)
	end
end	

function HealiumUnitFrames_Button_OnHide(self)
	self:GetParent():GetParent():StopMovingOrSizing()		
	local unit = self.TargetUnit
	if unit then
		if Healium_Units[unit] then
			for i,v in ipairs(Healium_Units[unit]) do
				if v == self then
					table.remove(Healium_Units[unit], i)
					break
				end
			end
		end
		self.TargetUnit = nil
	end
end	

function HealiumUnitFrames_Button_OnMouseDown(self, button)
	if button == "RightButton" then
		self:GetParent():GetParent():StartMoving()	
	end
end

function HealiumUnitFrames_Button_OnMouseUp(self, button)
	if button == "RightButton" then
		self:GetParent():GetParent():StopMovingOrSizing()		
	end	
end

function Healium_HideAllFrames()
	PartyFrame:Hide()
	PetsFrame:Hide()
	MeFrame:Hide()
	Healium_HideAllRaidFrames()
end

function Healium_ShowHidePartyFrame()
	if Healium.ShowPartyFrame then
		PartyFrame:Show()
	else
		PartyFrame:Hide()
	end
end

function Healium_ShowHidePetsFrame()
	if Healium.ShowPetsFrame then
		PetsFrame:Show()
	else
		PetsFrame:Hide()
	end
end

function Healium_ShowHideMeFrame()
	if Healium.ShowMeFrame then
		MeFrame:Show()
	else
		MeFrame:Hide()
	end
end

function Healium_ShowHideGroupFrame(group)
	if Healium.ShowGroupFrames[group] then
		GroupFrames[group]:Show()
	else
		GroupFrames[group]:Hide()
	end
end

function Healium_HideAllRaidFrames()
	for i,j in ipairs(GroupFrames) do
		j:Hide()
	end
end

function Healium_ShowAllRaidFramesWithMembers()
end
		
function Healium_Show10ManRaidFrames()
	GroupFrames[1]:Show()
	GroupFrames[2]:Show()
end

function Healium_Show25ManRaidFrames()
	for i=1, 5, 1 do
		GroupFrames[i]:Show()
	end
end

function Healium_Show40ManRaidFrames()
	for i=1, 8, 1 do
		GroupFrames[i]:Show()
	end
end

function Healium_CreateUnitFrames()
	PartyFrame = CreatePartyUnitFrame("HealiumPartyFrame", "Party")
	if Healium.ShowPartyFrame then
		PartyFrame:Show()
	end
	
	PetsFrame = CreatePetUnitFrame("HealiumPetFrame", "Pets")
	if Healium.ShowPetsFrame then
		Healium_DebugPrint(Healium.ShowPetsFrame)
		PetsFrame:Show()
	end
	
	MeFrame = CreateMeUnitFrame("HealiumMeFrame", "Me")
	if Healium.ShowMeFrame then
		MeFrame:Show()
	end	

	for i=1, 8, 1 do
		GroupFrames[i] = CreateGroupUnitFrame("HealiumGroup" .. i .. "Frame", "Group " .. i, tostring(i))
	end	
end


function Healium_SetScale()
	local Scale = Healium.Scale
	
	PartyFrame:SetScale(Scale)
	PetsFrame:SetScale(Scale)	
	MeFrame:SetScale(Scale)
	
	for i,j in ipairs(GroupFrames) do
		j:SetScale(Scale)
	end	
end

function Healium_UpdateBuffs(unit, frame)

	local buffIndex = 1
	
	if Healium.ShowBuffs then
		for i=1, 100, 1 do
			local name, rank, icon, count, debuffType, duration, expirationTime, source, isStealable = UnitBuff(unit, i, true)
			if name  then 
				if (duration > 0) and (source == "player") then
					
					local armed = false
					
					for j=1, Healium.ButtonCount, 1 do
						if HealiumDropDownButton[j] == name then
							armed = true
							break
						end
					end
					
					if armed == true then
						local buffFrame = frame.buffs[buffIndex]

						buffFrame.icon:SetTexture(icon)
						
						if count > 1 then
							buffFrame.count:SetText(count)
							buffFrame.count:Show()
						else
							buffFrame.count:Hide()
						end
						
						if duration and duration > 0 then
							local startTime = expirationTime - duration
							buffFrame.cooldown:SetCooldown(startTime, duration)
							buffFrame.cooldown:Show()
						else
							buffFrame.cooldown:Hide()
						end
						
						buffFrame:Show()
						buffIndex = buffIndex + 1					
						if buffIndex > MaxBuffs then
							break
						end			
					end
				end
			else
				break
			end
		end
	end

	-- hide remainder frames
	for i = buffIndex, MaxBuffs, 1 do
		frame.buffs[i]:Hide()
	end

end


